.module GUI

.var ubyte, DirOffset   ; In-column.
.var ubyte, FileOffset  ; In-column.
.var ubyte, SelectedCol ; Which col selected?

.var ubyte, DirScrollOffset
.var ubyte, FileScrollOffset

.var ubyte, AvailableDirs
.var ubyte, AvailableFiles

.var uword, DirNames
.var uword, FileNames

; ---------------------------------------------------------
; GetSelectionOffset -> Get selection rectangle offset.
; ---------------------------------------------------------
; Inputs:    a = list position.
; Outputs:   hl = offset (add gbuf).
;            c = 7
; Destroyed: af, b, de.
; ---------------------------------------------------------
GetSelectionOffset
	add a,a
	ld b,a
	add a,a
	add a,b
	ld l,a
	ld h,0
	add hl,hl
	add hl,hl
	push hl
	add hl,hl
	pop de
	add hl,de
	ld c,7
	ret
	

; ---------------------------------------------------------
; Reset -> Set up for defaults.
; ---------------------------------------------------------
; Destroyed: af, hl.
; ---------------------------------------------------------
Reset
	xor a
	ld (DirOffset),a
	ld (FileOffset),a
	ld (SelectedCol),a
	ld (AvailableDirs),a
	ld (AvailableFiles),a
	ld (DirScrollOffset),a
	ld (FileScrollOffset),a
	ld hl,saveSScreen
	ld (FileNames),hl
	ld hl,saveSScreen+416
	ld (DirNames),hl
	ret


; ---------------------------------------------------------
; Draw -> Draw the GUI interface.
; ---------------------------------------------------------
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
Draw

	ld hl,DirOffset
	ld de,DirScrollOffset
	ld a,(AvailableDirs)
	ld b,a
	call FixScroll

	ld hl,FileOffset
	ld de,FileScrollOffset
	ld a,(AvailableFiles)
	ld b,a
	call FixScroll

	ld hl,Back
	ld de,plotSScreen
	ld bc,768
	ldir
	
	; Draw files/folders
	ld a,(AvailableDirs)
	or a
	jr z,NoDirs
	cp 8
	jr c,{+}
	ld a,8
+	ld b,a

	ld hl,1+8*256
	ld (penCol),hl	
	ld hl,(DirNames)
	ld a,(DirScrollOffset)
	push bc
	call FindNthString
	pop bc
	
-	push bc
	call Parent.FixedWidth.PutS
	ld a,(penRow)
	add a,6
	ld (penRow),a
	ld a,1
	ld (penCol),a
	pop bc
	djnz {-}
	
NoDirs
	
	; Draw files/folders
	ld a,(AvailableFiles)
	or a
	jr z,NoFiles
	cp 8
	jr c,{+}
	ld a,8
+	ld b,a

	ld hl,40+8*256
	ld (penCol),hl
	
	ld hl,(FileNames)
	ld a,(FileScrollOffset)
	push bc
	call FindNthString
	pop bc

-	push bc
	call Parent.FixedWidth.PutS
	ld a,(penRow)
	add a,6
	ld (penRow),a
	ld a,40
	ld (penCol),a
	pop bc
	djnz {-}
	
NoFiles
	
	; Draw directory selection rectangle.
	
	ld a,(SelectedCol)
	or a
	jr nz,FileSelected
	ld a,(DirOffset)
	call GetSelectionOffset
	
	ld de,plotSScreen+12*7
	add hl,de
	ld de,8
	
--	ld a,(hl)
	xor %01111111
	ld (hl),a
	inc hl
	ld b,3
-	ld a,(hl)
	cpl
	ld (hl),a
	inc hl
	djnz {-}
	ld a,(hl)
	xor %11000000
	ld (hl),a
	add hl,de
	dec c
	jr nz,{--}
	jr DoneDirSelected
	
FileSelected
	; Draw file selection rectangle.
	ld a,(FileOffset)
	call GetSelectionOffset
	
	ld de,plotSScreen+12*7+5
	add hl,de
	ld de,6
	
--	ld b,6
-	ld a,(hl)
	cpl
	ld (hl),a
	inc hl
	djnz {-}
	ld a,(hl)
	xor %10000000
	ld (hl),a
	add hl,de
	dec c
	jr nz,{--}
DoneDirSelected


	ret
	

; hl->offset
; de->scroll
; b = max
FixScroll
	ld a,b
	or a
	jr nz,{+}
	ld (de),a
	ld (hl),a
	ret
+
	ld a,(de)
	sub b
	neg
	cp (hl)
	jr nz,{+}
	dec a
	ld (hl),a
	ret

+

	ld a,(hl)
	cp 8
	jr nz,{+}
	dec (hl)
	
	ld a,(de)
	add a,(hl)
	cp b
	ret z
	ld a,(de)
	inc a
	ld (de),a
	ret	
+	cp -1
	ret nz
	inc (hl)
	
	ld a,(de)
	or a
	ret z
	dec a
	ld (de),a
	ret
	


; ---------------------------------------------------------
; RefreshFileList -> Get file listing from drive.
; ---------------------------------------------------------
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
RefreshFileList

	call Parent.Dialogs.Dither
	ld a,Parent.Resources.Dialogs.Working
	call ShowDialog


	xor a
	ld (DirOffset),a
	ld (FileOffset),a
	ld (AvailableDirs),a
	ld (AvailableFiles),a
	; File = 9
	; Dir = 13

	ld hl,(FileNames)
	ld de,(DirNames)
	ld bc,32+32*256
	call Parent.VMusic2.Directory.GetList
	ret nz
	
	ld a,b
	ld (AvailableFiles),a
	ld a,c
	ld (AvailableDirs),a
	
	xor a
	ret

; ---------------------------------------------------------
; GetSelectedName -> Get the selected item's name.
; ---------------------------------------------------------
; Outputs:   OP1 = name.
; ---------------------------------------------------------
GetSelectedName
	ld a,(SelectedCol)
	or a
	jr nz,UseFileName

	ld hl,(DirNames)
	ld a,(DirOffset)
	ld b,a
	ld a,(DirScrollOffset)
	jr UsedDirName
UseFileName
	ld hl,(FileNames)
	ld a,(FileOffset)
	ld b,a
	ld a,(FileScrollOffset)
UsedDirName
	add a,b
	ld b,a
	or a
	jr z,{+}
	
-	ld a,(hl)
	inc hl
	or a
	jr nz,{-}
	djnz {-}
	
+	ld de,OP1

-	ld a,(hl)
	ld (de),a
	inc hl
	inc de
	or a
	jr nz,{-}
	ret


FindNthString
	or a
	ret z
	ld b,a
-	ld a,(hl)
	inc hl
	or a
	jr nz,{-}
	djnz {-}
	ret

; ---------------------------------------------------------
; ShowDialogDitherDelay -> Show a dialog on a dithered
;                          background and wait for a key.
; ---------------------------------------------------------
; Inputs:   a = dialog index.
; ---------------------------------------------------------
ShowDialogDitherDelay
	push af
	call Parent.Dialogs.Dither
	pop af
	; Run on...

; ---------------------------------------------------------
; ShowDialogDelay -> Show a dialog and wait for a key.
; ---------------------------------------------------------
; Inputs:   a = dialog index.
; ---------------------------------------------------------
ShowDialogDelay
	call ShowDialog
	jp Parent.Dialogs.WaitKey

; ---------------------------------------------------------
; ShowDialog -> Show a dialog.
; ---------------------------------------------------------
; Inputs:   a = dialog index.
; ---------------------------------------------------------
ShowDialog
	ld hl,Parent.Resources.Dialogs
	call Parent.Dialogs.DrawMessageFromTable
	call ionFastCopy
	ret	
	
Back .incbmp "Resources/Mockup.gif"

.endmodule