.module Directory
.using Parent
; ---------------------------------------------------------
; MoveUp -> Move up to the parent directory.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = changed successfully; nz = failed.
; Destroyed: af, bc, de, hl, OP1.
; ---------------------------------------------------------
MoveUp
	ld a,'.'
	ld (OP1+0),a
	ld (OP1+1),a
	xor a
	ld (OP1+2),a

; ---------------------------------------------------------
; Change -> Change directory
; ---------------------------------------------------------
; Inputs:    OP1 = directory name to change to.
; Outputs:   z = changed successfully; nz = failed.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
Change
	ld a,Commands.Directory.Change
	jp IO.WriteNamedCommandOP1

; ---------------------------------------------------------
; Create -> Create a directory
; ---------------------------------------------------------
; Inputs:    OP1 = directory name to create.
; Outputs:   z = created successfully; nz = failed.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
Create
	ld a,Commands.Directory.Create
	jp IO.WriteNamedCommandOP1
	
; ---------------------------------------------------------
; Delete -> Delete a directory
; ---------------------------------------------------------
; Inputs:    OP1 = directory name to delete.
; Outputs:   z = deleted successfully; nz = failed.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
Delete
	ld a,Commands.Directory.Delete
	jp IO.WriteNamedCommandOP1


; ---------------------------------------------------------
; CheckIsDirectory -> Check if an item is a directory.
; ---------------------------------------------------------
; Inputs:    hl -> start of listing for the item.
; Outputs:   z = is directory; nz = is file.
; Destroyed: af.
; ---------------------------------------------------------
CheckIsDirectory
	push hl
-	ld a,(hl)
	cp '\r'
	jr z,{+}
	inc hl
	jr {-}
+	dec hl
	ld a,(hl)
	cp 'R'
	jr nz,{+}
	dec hl
	ld a,(hl)
	cp 'I'
	jr nz,{+}
	dec hl
	ld a,(hl)
	cp 'D'
	jr nz,{+}
	ld a,(hl)
	cp ' '
+	pop hl
	ret


; ---------------------------------------------------------
; GetList -> Get a directory listing.
; ---------------------------------------------------------
; Inputs:    hl -> storage for file listing.
;            b  =  maximum number of files to list.
;            de -> storage for directory listing.
;            c  =  maximum number of directories to list.
; Outputs:   z = success; nz = failure.
;            b = number of files found.
;            c = number of directories found.
; Destroyed: af, bc, de, hl, OP1-OP6.
; ---------------------------------------------------------
GetList


	.struct FileOffsets
		.var ubyte, Maximum
		.var ubyte, Found
	.endstruct
	
	.var uword, Files
	.var uword, Directories
	.var FileOffsets, NumFiles
	.var FileOffsets, NumDirectories

	ld (Files),hl
	ld (Directories),de
	ld a,b
	ld (NumFiles.Maximum),a
	ld a,c
	ld (NumDirectories.Maximum),a
	xor a
	ld (NumFiles.Found),a
	ld (NumDirectories.Found),a

	ld a,Commands.Directory.List
	call IO.WriteShortByteCR
	ret nz

	; Skip first result	
	call IO.ReadOP1
	ret nz

GetNextItem
	call IO.ReadOP1
	ret nz

	dec hl
	dec hl
	ld a,(hl)
	cp '>'
	jr nz,{+}
	
	; Run out of files!
	ld a,(NumFiles.Found) \ ld b,a
	ld a,(NumDirectories.Found) \ ld c,a
	
	xor a
	ret
+
	
	
	; Was it a directory or a file?
	ld a,(hl)
	cp 'R'
	jr nz,IsFile
	dec hl

	ld a,(hl)
	cp 'I'
	jr nz,IsFile
	dec hl

	ld a,(hl)
	cp 'D'
	jr nz,IsFile
	dec hl

	ld a,(hl)
	cp ' '
	jr nz,IsFile
	dec hl
	
	ld a,(hl)
	cp '.'
	jr nz,IsDirectory
	ld de,OP1
	or a
	sbc hl,de
	jr z,FoundEnoughOfItem

IsDirectory
	ld de,Directories
	ld hl,NumDirectories
	jr DetectedType
IsFile
	ld de,Files
	ld hl,NumFiles
DetectedType
	ld a,(hl)
	or a
	jr z,FoundEnoughOfItem
	
	dec a
	ld (hl),a
	inc hl
	inc (hl)
	
	; Hm :)
	ex de,hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	dec hl
	ex de,hl
	
	ld hl,OP1
	
-	ld a,(hl)
	cp ' '
	jr z,FoundFullName
	cp '\r'
	jr z,FoundFullName
	ld (bc),a
	inc hl
	inc bc
	jr {-}	
	
FoundFullName
	xor a
	ld (bc),a
	inc bc
	ex de,hl
	ld (hl),c
	inc hl
	ld (hl),b	
FoundEnoughOfItem
	jr GetNextItem
	
.endmodule