.module Music
.using Parent

; ---------------------------------------------------------
; Play -> Plays an MP3 file.
; ---------------------------------------------------------
; Inputs:    OP1 = filename.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
Play
	ld a,Commands.Music.Play
	jp IO.WriteNamedCommandOP1
	
; ---------------------------------------------------------
; Stop -> Stops playing the MP3 file.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
Stop
	ld a,Commands.Music.Stop
	jp IO.WriteShortByteCR

; ---------------------------------------------------------
; PlayAll -> Play all files with .MP3 extension.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
PlayAll
	ld a,Commands.Music.PlayAll
	jp IO.WriteShortByteCR

; ---------------------------------------------------------
; SkipNext -> Skip to next track.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
SkipNext
	ld a,Commands.Music.Next
	jp IO.WriteShortByteCR

; ---------------------------------------------------------
; SkipPrevious -> Skip to previous track.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
SkipPrevious
	ld a,Commands.Music.Previous
	jp IO.WriteShortByteCR

; ---------------------------------------------------------
; WriteRegister -> Write to a register.
; ---------------------------------------------------------
; Inputs:    a  = register number.
;            hl = register value.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, OP1.
; ---------------------------------------------------------
WriteRegister
	ld (OP1+0),a
	ld (OP1+1),hl
	xor a
	ld (OP1+3),a
	ld a,Commands.Music.WriteReg
	jp IO.WriteNamedCommandOP1

; ---------------------------------------------------------
; ReadRegister -> Read from a register.
; ---------------------------------------------------------
; Inputs:    a = register number.
; Outputs:   z  = success; nz = failure.
;            hl = register value.
; Destroyed: af, bc, de, OP1.
; ---------------------------------------------------------
ReadRegister
	ld (OP1+0),a
	xor a
	ld (OP1+1),a
	ld a,Commands.Music.ReadReg
	call IO.WriteShortByteCR
	ret nz
	call IO.ReadOP1
	ld hl,(OP1+0)
	ret

; ---------------------------------------------------------
; GetVolume -> Set the volume.
; ---------------------------------------------------------
; Outputs:   z = success; nz = failure.
;            h = left speaker volume.
;            l = right speaker volume.
; Destroyed: af, bc, de, OP1.
; ---------------------------------------------------------
GetVolume
	ld a,$0B
	jp ReadRegister

; ---------------------------------------------------------
; SetVolume -> Set the volume.
; ---------------------------------------------------------
; Inputs:    h = left speaker volume.
;            l = right speaker volume.
; Outputs:   z = success; nz = failure.
; Destroyed: af, bc, de, OP1.
; ---------------------------------------------------------
SetVolume
	ld a,$0B
	jp WriteRegister
	

.endmodule