.module Utility
.using Parent

; ---------------------------------------------------------
; SetUInt32OP1 -> Copy an unsigned 32-bit integer to OP1
; ---------------------------------------------------------
; Inputs:    dehl = 32-bit value.
; Outputs:   OP1 = value.
; Destroyed: af, bc, de, hl, OP2.
; ---------------------------------------------------------
SetUInt32OP1
	push hl
	push de
	
	; First up, preload OP1 with 65536
	ld hl,C65536
	rst rMOV9TOOP1
	
	pop hl ; hl => $de..
	bcall(_SetXXXXOP2)
	bcall(_FPMult)
	
	pop hl; hl => $..hl
	bcall(_SetXXXXOP2)
	bcall(_FPAdd)

	ret
C65536
.dvar tifloat, 65536

; ---------------------------------------------------------
; FormatSize -> Format a size in bytes into KB/MB/GB.
; ---------------------------------------------------------
; Inputs:    dehl = 32-bit value (size in bytes).
; Outputs:   hl = whole number.
;            c  = fractional part (/256).
;            b  = suffix (0 = B, 1 = KB, 2 = MB, 3 = GB).
; Destroyed: af.
; ---------------------------------------------------------
FormatSize
	ld b,0
FormatSizeLoop
	ld a,d
	or e
	jr nz,TooBig

	ld a,h
	and ~$03
	jr nz,TooBig

	ret

TooBig
	inc b
	; dehl > 1024
	; So, divide by 1024!
	
	; Divide by 256:
	ld c,l
	ld l,h
	ld h,e
	ld e,d
	ld d,0
	
	; /2->512
	srl e
	rr h
	rr l
	rr c

	; /2->1024
	srl e
	rr h
	rr l
	rr c

	jr FormatSizeLoop

; ---------------------------------------------------------
; FormatSizeOP1 -> Format a size in bytes into KB/MB/GB.
; ---------------------------------------------------------
; Inputs:    dehl = 32-bit value (size in bytes).
; Outputs:   OP1 = string representation of value.
; Destroyed: af, OP2-OP6.
; Remarks:   Maximum size = 1023.99MB = 9 chars.
; ---------------------------------------------------------
FormatSizeOP1
	call FormatSize
	push bc
	bcall(_SetXXXXOP2)
	bcall(_OP2ToOP1)
	ld a,6
	bcall(_FormEReal)
	bcall(_OP3ToOP1)

	; Append the '.'
	
	ld hl,OP1
-	ld a,(hl)
	or a
	jr z,{+}
	inc hl
	jr {-}
+
	ld (hl),'.'
	inc hl
	
	; Append the fractional part:
	pop bc
	push bc
	
	push hl
	; *100 = *4+*32+*64
	ld l,c
	ld h,0
	add hl,hl
	add hl,hl
	push hl
	add hl,hl
	add hl,hl
	add hl,hl
	push hl
	add hl,hl
	pop de
	add hl,de
	pop de
	add hl,de
	; h = fractional part

	ld a,h
	ld bc,10*256
-	cp b
	jr c,{+}
	sub b
	inc c
	jr {-}
+	ld b,a


	pop hl
	ld a,c
	add a,'0'
	ld (hl),a
	inc hl
	ld a,b
	add a,'0'
	ld (hl),a
	inc hl
	
	; Finally, K, M, G?
	pop bc
	inc b
	ld de,Power-1
	
-	inc de
	djnz {-}
	
	ld a,(de)
	ld (hl),a
	inc hl
	cp 'B'
	jr z,{+}
	ld (hl),'B'
	inc hl
+	ld (hl),0	
	ret

Power .db "BKMGT"
	
.endmodule